#include "SDL/SDL.bi"
#include "GL/gl.bi"
#include "GL/glext.bi"
#include "GL/glu.bi"
#include "AL/al.bi"
#include "AL/alut.bi"

#define _bool_ byte
#define _notex_ GLBindTexture GL_TEXTURE_2D, 0

'declara constantes
const as integer w = 1024
const as integer h = 600
const as GLfloat aspect_ratio = w/h
const false = 0
const true = not false
const as double pi = 3.1415926
const as double to_rad = (pi * 2) / 360
const as double MAX_RINGS = 100
const as double MAX_ESTRELAS_BG = 1024
const as double MAX_ESTRELAS_FG = 128

dim shared as double _dx_, _dy_
dim shared as _bool_ _3D_ON = false
dim shared as _bool_ SFX_ON = true
dim shared as _bool_ MUSICA_ON = true

'módulos
type coord3d
	x as double
	y as double
	z as double
end type

type hsvVal
	h as double
	s as double
	v as double
end type

#include "palete.bi"
#include "util.bi"
#include "framerate.bi"
#include "controle.bi"
#include "textura.bi"
#include "som.bi"

type ring
	ang_hor as double
	ang_ver as double
	cont1 as integer		'direção
	cont2 as integer		'objetos
	cont3 as integer		'ângulo
	cont4 as integer		'cor e sombra
	objeto as integer		'0: vazio
							'1: funil vertical
							'2: funil horizontal
							'3: funil giratório
							'4: shield
							'5: hover
							'6: energy bonus
							'7: mistério
	obj_ang as double
	
	declare sub novo
end type

type ringList
	r (0 to MAX_RINGS - 1) as ring
	
	declare sub nova
	declare sub push
	declare sub pinta
	declare sub atualiza
end type

type player
	z as double 'offset	
	vel as double
	acc as double
	ang as double
	angacc as double
	ang_hor as double
	ang_ver as double
	antigrav as _bool_
		
	particula_z as double
	flash as double '0-1
	
	shield as integer
	hover as double
	energia as double
	energia_vis as double
	min_vel as double
	
	declare sub novo
	declare sub pinta
	declare sub pintaHud
	declare sub controle
	declare sub fisica
	declare sub colisao
	declare sub sons
end type

type starList
	private:
		bg (0 to MAX_ESTRELAS_BG - 1) as coord3d
		fg (0 to MAX_ESTRELAS_FG - 1) as coord3d
	
	public:	
		declare sub nova
		declare sub pinta
		declare sub atualiza
end type

type mensagem
	private:	
		txt as string
		tempo as double
		
	public:
		declare sub nova (txt as string)
		declare sub atualiza
		declare sub pinta
end type

type graficos
	private:
		declare sub criaFrustum (sentido as integer)
		declare sub pintaCena
								
	public:
		estrela as textura
		pad as textura
		ship as textura
		sombra as textura
		particula as textura
		anel as textura
		hud as textura
		aviso as textura
		planeta as textura
		asteroides as textura
				
		declare sub carrega
		declare sub pinta
		declare sub ilumina (fonte as coord3d, amb as single)
		declare sub menuHeader
		declare sub menuFooter
		declare sub destroi
end type

type menuObj
	private:
		declare sub topo
		declare sub escolheCurso
		declare sub escolheNave
		
		cursor as integer
		planeta_ang as double
		planeta_quad as GLUquadricObj ptr
		
	public:
		declare sub main
end type

declare sub inicio
declare sub carregaJogo
declare sub jogo
declare function controles () as _bool_
declare sub pausa
declare sub fim

dim shared p as player
dim shared tunel as ringList
dim shared estrelas as starList
dim shared msg as mensagem
dim shared fnt as fonte
dim shared gfx as graficos
dim shared menu as menuObj
dim shared as somBuffer motor, rumble
dim shared som as somEngine
dim shared seq as sequenciador

'inicializa tudo
inicio
carregaJogo

'fluxo
menu.main

'-----------------------------------------------------------------------
sub inicio
	'inicia uma janela
	dim video as SDL_Surface ptr
	video = SDL_SetVideoMode(w, h, 16, SDL_OPENGL or SDL_DOUBLEBUF) 'or SDL_FULLSCREEN)

	if video = 0 then
		SDL_Quit
		end
	end if
		
	dim as string titulo_utf8 = "Space"
	SDL_WM_SetCaption titulo_utf8, titulo_utf8
	SDL_ShowCursor SDL_DISABLE
	
	'configura visual
	glClearColor 0, 0, 0, 1
	glEnable GL_TEXTURE_2D
	glPolygonMode GL_FRONT_AND_BACK, GL_FILL
	glEnable GL_BLEND
	glBlendFunc GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA
	glDisable GL_CULL_FACE
	glDisable GL_DEPTH_TEST
	glEnable GL_LINE_SMOOTH
	glDisable GL_POINT_SMOOTH
	glEnable GL_POINT_SPRITE
	glPointSize 4
		
	glEnable GL_LIGHT0 
	glEnable GL_NORMALIZE
	glEnable GL_COLOR_MATERIAL
	glDisable GL_LIGHTING	

	'testes
	dim as GLint range(0 to 1)
	glGetIntegerv GL_SMOOTH_LINE_WIDTH_RANGE, @range(0)
	
	print range(0), range(1)
	
	'tamanho da fonte
	if w < 640 then
		_dx_ = 6
		_dy_ = 9
	elseif w < 1280 or h < 600 then
		_dx_ = 12
		_dy_ = 18
	else
		_dx_ = 18
		_dy_ = 27
	end if
	
	'inicia som
	alutInit 0, 0
end sub

sub carregaJogo
	fps.setMaxFPS 0

	fnt.carrega "gfx/fonte.bmp"
	gfx.carrega

	motor.carrega "sfx/motor.wav"
	rumble.carrega "sfx/rumble.wav"

	seq.inicia
		
	som.cria
	som.defineVoz 0, motor
	som.defineVoz 1, rumble
end sub

sub fim
	seq.destroi
	fnt.destroi
	gfx.destroi
	
	motor.destroi
	rumble.destroi

	SDL_Quit
	alutExit
	end
end sub

sub jogo
	randomize timer
	tunel.nova
	estrelas.nova
	p.novo
	
	fps.reinicia
	seq.reinicia
	som.startYourEngines

	msg.nova "get ready!"

	'main loop
	do
		fps.atualiza
		kev.atualiza
		
		'controles
		if not controles then exit do
		p.controle
		
		'movimento
		p.fisica
		estrelas.atualiza
		tunel.atualiza		
	
		'gráficos
		gfx.pinta		
		
		'áudio
		p.sons
		seq.toca
		
		'atualiza tela
		SDL_GL_SwapBuffers
		
		'delay
		fps.preciseSleep
	loop
	
	som.para
end sub

function controles () as _bool_
	if kev.confereTecla(kev.K_3D, kev.PRESSED) then 		_3D_ON = not _3D_ON
	if kev.confereTecla(kev.K_MUSICA, kev.PRESSED) then		MUSICA_ON = not MUSICA_ON
			
	if kev.confereTecla(kev.K_SFX, kev.PRESSED) then
		if not SFX_ON then
			SFX_ON = true
			som.startYourEngines
		else
			SFX_ON = false
			som.para
		end if
	end if
	
	if kev.confereTecla(kev.K_PAUSA, kev.PRESSED) then 		pausa
	if kev.confereTecla(kev.K_RETORNA, kev.PRESSED) then 	return false
	
	return true
end function

sub pausa
	som.para

	do
		fps.atualiza
		kev.atualiza
				
		sleep 20
	loop until kev.confereTecla(kev.K_RETORNA, kev.PRESSED) or kev.confereTecla(kev.K_PAUSA, kev.PRESSED)
	
	som.startYourEngines
end sub

'-----------------------------------------------------------------------
sub graficos.carrega
	estrela.carrega "gfx/estrela.bmp", false, false, false
	pad.carrega "gfx/pad.bmp"
	ship.carrega "gfx/ship.bmp"
	sombra.carrega "gfx/sombra.bmp"
	particula.carrega "gfx/particula.bmp"
	anel.carrega "gfx/anel.bmp", true, true, false
	hud.carrega "gfx/hud.bmp", false, false, false
	aviso.carrega "gfx/aviso.bmp"
	planeta.carrega "gfx/planeta.bmp"
	asteroides.carrega "gfx/asteroides.bmp"
end sub

sub graficos.destroi
	estrela.destroi
	pad.destroi
	ship.destroi
	sombra.destroi
	particula.destroi
	anel.destroi
	hud.destroi
	aviso.destroi
	planeta.destroi
	asteroides.destroi
end sub

sub graficos.pinta
	'configura projeção
	glClear GL_COLOR_BUFFER_BIT
	glViewport 0, 0, w, h
	
	'cena 3D
	dim as integer i, s, f
	
	if _3D_ON then
		s = -1
		f = 1
	else
		s = 0
		f = 0
	end if
	
	for i = s to f step 2
		glClear GL_DEPTH_BUFFER_BIT
	
		select case i
			case 0: glColorMask GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE
			case 1: glColorMask GL_TRUE, GL_FALSE, GL_FALSE, GL_FALSE
			case -1: glColorMask GL_FALSE, GL_FALSE, GL_TRUE, GL_FALSE
		end select
		
		glPushMatrix
			glMatrixMode GL_PROJECTION
			glLoadIdentity
			criaFrustum i
			
			glMatrixMode GL_MODELVIEW
			glLoadIdentity
			gluLookAt 0, -0.2, -0.5, 0, -0.2, 1, sin(p.ang) * -0.5, cos(p.ang), 0
			
			pintaCena
		glPopMatrix
	next
	
	'texto
	glColorMask GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE
	
	fnt.orto
	'fnt.setCor 1, 1, 1
	'fnt.setSombra true
	'fnt.pintaTopoDir fps.getFPS, 1
	
	p.pintaHud
	
	msg.atualiza
	msg.pinta
	
	fnt.restaura
end sub

sub graficos.criaFrustum (sentido as integer)
	const as double near = 0.5
	const as double far = 200
	const as double foco = 8
	const as double separacao = 1
	const as double fov = 60
	const as double DTR = 0.0174532925
	
	dim as double ftop = near * tan(DTR * fov / 2)
	dim as double fright = aspect_ratio * ftop
	dim as double fshift = (separacao / 2) * near / foco
	
	glFrustum -fright + (fshift * sentido), fright + (fshift * sentido), -ftop, ftop, near, far
end sub

sub graficos.pintaCena
	'reposiciona luz
	dim fonte as coord3d
	fonte.x = cos(p.ang_hor) * 10
	fonte.y = 0
	fonte.z = sin(p.ang_hor) * 10
	
	ilumina fonte, 0.4 

	'gráficos
	estrelas.pinta

	glLineWidth 2
	tunel.pinta
	
	p.pinta
end sub

sub graficos.ilumina (fonte as coord3d, amb as single)
	dim l_amb(0 to 3) as single => {0, 0, 0, 1} 
	dim l_diff(0 to 3) as single => {1, 1, 1, 1}
	dim l_spec(0 to 3) as single => {0, 0, 0, 1}
	dim l_pos(0 to 3) as single => {0, 10, 10, 1}
	
	l_pos(0) = fonte.x
	l_pos(1) = fonte.y
	l_pos(2) = fonte.z
	
	dim as integer i
	for i = 0 to 2
		l_amb(i) = amb
		l_diff(i) = 1 - amb
	next i
	
	glLightfv GL_LIGHT0, GL_AMBIENT, @l_amb(0)
	glLightfv GL_LIGHT0, GL_DIFFUSE, @l_diff(0)
	glLightfv GL_LIGHT0, GL_SPECULAR, @l_spec(0)
	glLightfv GL_LIGHT0, GL_POSITION, @l_pos(0)
end sub

sub graficos.menuHeader
	'projeção, câmera e rotinas comuns a todas as cenas de intro/menu
	fps.atualiza
	kev.atualiza
	
	glClear GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT
		
	glMatrixMode GL_PROJECTION
	glLoadIdentity
	criaFrustum 0
		
	glMatrixMode GL_MODELVIEW
	glLoadIdentity
	gluLookAt 0, 0, -1.5, 0, 0, 0, 0, 1, 0
end sub

sub graficos.menuFooter
	'atualiza tela
	SDL_GL_SwapBuffers
		
	'delay
	fps.preciseSleep
end sub

'-----------------------------------------------------------------------
sub ring.novo
	ang_hor = 0
	ang_ver = 0
	cont1 = 0
	cont2 = 0
	cont3 = 0
	cont4 = 0
	objeto = 0
	obj_ang = 0
end sub

sub ringList.nova
	dim i as integer
	
	for i = 0 to MAX_RINGS - 1
		r(i).novo
		r(i).cont2 = i mod 12
		r(i).cont4 = i mod 32 
	next i
	
	'for i = 0 to MAX_RINGS - 1
	'	push
	'next i
end sub

sub ringList.push
	dim i as integer
	
	for i = 0 to MAX_RINGS - 2
		r(i) = r(i + 1)
	next i
	
	r(MAX_RINGS - 1).novo
	
	'mudança de direção
	if r(MAX_RINGS - 2).cont1 > 10 and rnd < 0.05 then
		r(MAX_RINGS - 1).ang_hor = ((1 - (rnd * 2)) * pi) / 50
		r(MAX_RINGS - 1).ang_ver = ((1 - (rnd * 2)) * pi) / 100
		r(MAX_RINGS - 1).cont1 = 0
	else
		r(MAX_RINGS - 1).ang_hor = r(MAX_RINGS - 2).ang_hor
		r(MAX_RINGS - 1).ang_ver = r(MAX_RINGS - 2).ang_ver
		r(MAX_RINGS - 1).cont1 = r(MAX_RINGS - 2).cont1 + 1
	end if
	
	'objetos
	if r(MAX_RINGS - 2).cont2 >= 12 then
		r(MAX_RINGS - 1).cont2 = 0
		
		'obstáculos
		if rnd < 0.1 then
			'funil horizontal
			if r(MAX_RINGS - 1).cont1 > 16 and abs(r(MAX_RINGS - 1).ang_hor) > 0.035 then
				r(MAX_RINGS - 1).objeto = 2
				
			'funil vertical
			else
				'estático
				if rnd < 0.7 then
					r(MAX_RINGS - 1).objeto = 1
					
				'giratório
				else
					r(MAX_RINGS - 1).objeto = 3
					r(MAX_RINGS - 1).obj_ang = rnd * 2 * pi
				end if
			end if
			
		'power-ups
		elseif rnd < 0.15 then
			dim as double prob (0 to 3)
			dim as double dificuldade = (0.75 - p.min_vel) * 1.4
			
			'distribui probabilidade
			for i = 0 to 3
				prob(i) = rnd
			next i
			
			'pesos e raridade
			prob(0) = prob(0) * dificuldade
			prob(1) = prob(1) * dificuldade * 1.5
			prob(2) = prob(2) * dificuldade * 2
			
			'não dá o que o jogador não precisa
			if p.shield = 5 or p.shield = 0 then prob(0) = 0
			if p.hover >= 4.9 then prob(1) = 0
			if p.energia = 2 then prob(2) = 0
			
			'aumenta o que o jogador precisa muito
			if p.shield = 1 then prob(0) = prob(0) * 3
			
			'encontra o maior valor
			dim as double maior_val = 0
			dim as integer maior = 0
			
			for i = 1 to 3
				if prob(i) > prob(maior) then
					maior_val = prob(i)
					maior = i
				end if
			next i
			
			r(MAX_RINGS - 1).objeto = 4 + maior
			r(MAX_RINGS - 1).obj_ang = angadd(rnd * pi, pi / -2)
		end if
	else
		r(MAX_RINGS - 1).cont2 = r(MAX_RINGS - 2).cont2 + 1
	end if
	
	'ângulo do anel
	if r(MAX_RINGS - 2).cont3 >= 8 then
		r(MAX_RINGS - 1).cont3 = 0
	else
		r(MAX_RINGS - 1).cont3 = r(MAX_RINGS -2).cont3 + 1
	end if
	
	'cores e sombras
	if r(MAX_RINGS - 2).cont4 >= 32 then
		r(MAX_RINGS - 1).cont4 = 0
	else
		r(MAX_RINGS - 1).cont4 = r(MAX_RINGS -2).cont4 + 1
	end if
end sub

sub ringList.pinta
	dim ringpos (0 to MAX_RINGS - 1) as coord3d
	dim soma_ang (0 to 1, 0 to MAX_RINGS - 1) as double '0: hor, 1: ver

	'posição inicial
	ringpos(0).x = 0
	ringpos(0).y = 0
	ringpos(0).z = p.z
	
	soma_ang(0, 0) = (pi / 2) + (r(0).ang_hor * p.z)
	soma_ang(1, 0) = (pi / 2) + (r(0).ang_ver * p.z)
		
	'calcula posições no espaço
	dim i as integer
	for i = 1 to MAX_RINGS - 1
		ringpos(i).x = ringpos(i - 1).x + (cos(soma_ang(0, i - 1)) * sin(soma_ang(1, i - 1)))
		ringpos(i).y = ringpos(i - 1).y + (cos(soma_ang(1, i - 1)))
		ringpos(i).z = ringpos(i - 1).z + (sin(soma_ang(0, i - 1)) * sin(soma_ang(1, i - 1)))
		
		soma_ang(0, i) = angadd(soma_ang(0, i - 1), r(i).ang_hor)
		soma_ang(1, i) = angadd(soma_ang(1, i - 1), r(i).ang_ver)
	next i
		
	'pinta anéis em ordem
	dim as integer res = 4
	
	_notex_
	for i = (MAX_RINGS - 1) to 0 step -1
		glPushMatrix
			'posiciona
			glTranslatef ringpos(i).x, ringpos(i).y, ringpos(i).z
			glRotatef 90 - (soma_ang(0, i) * 180 / pi), 0, 1, 0
			glRotatef 90 + (soma_ang(1, i) * 180 / pi), 1, 0, 0
			
			'anel
			dim as double alfa = (MAX_RINGS - i) / MAX_RINGS
			
			if i < (MAX_RINGS * 0.5) then res = 8
			
			dim d as double
			if r(i).ang_hor = 0 and r(i).ang_ver = 0 then
				'launch pad
				dim as single sombra = (((abs(r(i).cont2 - 6) / 6) * 0.25) + 0.75) * alfa
				hsv 0, 0, sombra
				
				gfx.pad.aplica
				glBegin GL_TRIANGLE_STRIP
					for d = 0 to (2 * pi) + (pi / res) step (pi / res)
						glTexCoord2f 0.3, (sin(d) / 2) + 0.5
						glVertex3f cos(d), sin(d), -0.2
						
						glTexCoord2f 0, (sin(d) / 2) + 0.5
						glVertex3f cos(d), sin(d), 1						
					next d
				glEnd				
				_notex_
			else
				'anel normal								
				glPushMatrix
					glRotatef r(i).cont3 / 9 * 360, 0, 0, 1
				
					dim as single sombra = ((abs(r(i).cont4 - 16) / 16) * 0.6) + 0.4
					
					'hsva (r(i).cont4 mod 32) * (360 / 32), 1, 1, alfa
					hsva pal.ring.h, pal.ring.s, sombra, alfa
				
					gfx.anel.aplica
					glBegin GL_QUADS
						glTexCoord2i 0, 0
						glVertex3f -1.1, -1.1, 0
						
						glTexCoord2i 0, 1
						glVertex3f -1.1, 1.1, 0
						
						glTexCoord2i 1, 1
						glVertex3f 1.1, 1.1, 0
						
						glTexCoord2i 1, 0
						glVertex3f 1.1, -1.1, 0
					glEnd
					_notex_
				glPopMatrix
			end if
			
			'objetos
			if r(i).objeto > 0 and r(i).objeto <= 3 then 'funil
				glPushMatrix
					setPalA pal.funil_fix, alfa
				
					'funil horizontal
					if r(i).objeto = 2 then				
						glRotatef 90, 0, 0, 1	
										
					'giratório
					elseif r(i).objeto = 3 then
						setPalA pal.funil_rot, alfa
						glRotatef r(i).obj_ang / to_rad, 0, 0, -1
					end if					
					
					glBegin GL_LINES
						dim d as double
						
						for d = -1 to -0.5 step 0.1 * (8 / res)
							glVertex3f d, 1, 0
							glVertex3f d, -1, 0
						next d
						
						for d = 0.5 to 1 step 0.1 * (8 / res)
							glVertex3f d, 1, 0
							glVertex3f d, -1, 0
						next d
						
						glVertex3f -1, -1, 0
						glVertex3f -0.5, -1, 0
						
						glVertex3f 0.5, -1, 0
						glVertex3f 1, -1, 0
						
						glVertex3f -1, 1, 0
						glVertex3f -0.5, 1, 0
						
						glVertex3f 0.5, 1, 0
						glVertex3f 1, 1, 0
					glEnd
				glPopMatrix
				
			elseif r(i).objeto >= 4 then 'power-ups
				glPushMatrix
					glRotatef r(i).obj_ang / to_rad, 0, 0, -1
					glScalef 0.2, 0.2, 0.2
					glTranslatef 0, 4, 0
				
					'diamante
					glPushMatrix				
						glRotatef ((i mod 18) + p.z) * 20, 1, 1, 0
						
						dim texto as string
						select case r(i).objeto
							case 4
								setPalA pal.shield, alfa
								texto = "^"
							case 5
								setPalA pal.hover, alfa
								texto = "&"
							case 6
								setPalA pal.energia, alfa
								texto = "%"
							case 7
								setPalA pal.misterio, alfa
								texto = "?"
						end select
						
						glBegin GL_LINE_STRIP
							glVertex3f -1, 0, 0
							glVertex3f 0, 1, 0
							glVertex3f 1, 0, 0
							glVertex3f 0, -1, 0
							glVertex3f -1, 0, 0
						glEnd
						
						glBegin GL_LINE_STRIP
							glVertex3f 0, 0, -1
							glVertex3f 0, 1, 0
							glVertex3f 0, 0, 1
							glVertex3f 0, -1, 0
							glVertex3f 0, 0, -1
						glEnd
						
						glBegin GL_LINE_STRIP
							glVertex3f 0, 0, -1
							glVertex3f 1, 0, 0
							glVertex3f 0, 0, 1
							glVertex3f -1, 0, 0
							glVertex3f 0, 0, -1
						glEnd
					glPopMatrix
					
					'texto
					if res >= 8 then
						glPushMatrix
							glScalef -1 / _dy_, 1 / _dy_, 0
							glTranslatef _dx_ * -0.5, _dy_ * -0.4, 0
							
							fnt.setCor 1, 1, 1
							fnt.setSombra false
							fnt.pinta texto, 0, 0
							_notex_
						glPopMatrix
					end if
				glPopMatrix
			end if
		glPopMatrix
	next i
end sub

sub ringList.atualiza
	dim i as integer
	
	for i = (MAX_RINGS - 1) to 0 step -1
		'movimento do funil giratório
		if r(i).objeto = 3 then
			r(i).obj_ang = angadd(r(i).obj_ang, 0.012 * fps.getFator)
		end if
	next
end sub

'-----------------------------------------------------------------------
sub starList.nova
	'gera listas
	dim i as integer
	
	for i = 0 to MAX_ESTRELAS_BG - 1
		bg(i).x = rnd * 360
		bg(i).y = rnd * 360
		bg(i).z = 0
	next i
	
	for i = 0 to MAX_ESTRELAS_FG - 1
		dim as double ang, raio
		ang = rnd * 2 * pi
		raio = 0.1 + (rnd * 0.9)
	
		fg(i).x = cos(ang) * raio
		fg(i).y = sin(ang) * raio
		fg(i).z = (rnd * 2) - 1
	next i
end sub

sub starList.pinta
	dim i as integer

	'bg
	glPushMatrix
		'simula skydome
		dim as double tx, ty
		tx = p.ang_hor / to_rad
		ty = p.ang_ver / to_rad
		
		'centraliza
		glTranslatef -180, -180, 80

		'estrelas
		gfx.estrela.aplica
		glTexEnvi GL_POINT_SPRITE, GL_COORD_REPLACE, GL_TRUE
		
		glBegin GL_POINTS		
			hsv 0, 0, 0.5
			for i = 0 to MAX_ESTRELAS_BG
				if i = MAX_ESTRELAS_BG / 2 then hsv 0, 0, 1

				dim as double x, y
				x = bg(i).x + tx
				y = bg(i).y + ty
				
				if x > 360 then x = x - 360
				if y > 360 then y = y - 360

				glVertex3f x, y, 0
			next i
		glEnd
	
		glTexEnvi GL_POINT_SPRITE, GL_COORD_REPLACE, GL_FALSE
	glPopMatrix
	
	'flash
	if p.flash > 0 then
		fnt.orto
		
		_notex_
		hsva 20, 0.9, 1, p.flash
		glBegin GL_QUADS
			glVertex2f 0, 0
			glVertex2f 0, h
			glVertex2f w, h
			glVertex2f w, 0
		glEnd
		
		fnt.restaura
	end if
	
	'fg
	scope
		dim as double tam = 0.003
		
		glPushMatrix
			glScalef 100, 100, 80
		
			gfx.particula.aplica
			glBegin GL_QUADS
				for i = 0 to MAX_ESTRELAS_FG - 1
					select case i mod 2
						case 0: hsva 0, 0, 1, 0.4
						case 1: hsva 0, 0, 1, 0.8
					end select
					
					glTexCoord2i 0, 0
					glVertex3f fg(i).x - tam, fg(i).y - tam, fg(i).z
					
					glTexCoord2i 0, 1
					glVertex3f fg(i).x - tam, fg(i).y + tam, fg(i).z
					
					glTexCoord2i 1, 1
					glVertex3f fg(i).x + tam, fg(i).y + tam, fg(i).z
					
					glTexCoord2i 1, 0
					glVertex3f fg(i).x + tam, fg(i).y - tam, fg(i).z
				next i
			glEnd
		glPopMatrix
	end scope
end sub

sub starList.atualiza
	dim i as integer
	
	for i = 0 to MAX_ESTRELAS_FG - 1
		dim as single vel = 0.02 + (0.01 * (i mod 5))
		
		fg(i).z = fg(i).z - (vel * p.vel * fps.getFator)
		if fg(i).z < -1 then fg(i).z = fg(i).z + 2
	next i
end sub

'-----------------------------------------------------------------------
sub mensagem.nova (txt as string)
	this.txt = txt
	tempo = 100
end sub

sub mensagem.atualiza
	tempo = tempo - fps.getFator
	if tempo < 0 then tempo = 0
end sub

sub mensagem.pinta
	if tempo = 0 then exit sub
	
	fnt.setSombra false
	fnt.setCor 1, 1, 1, tempo / 100
	fnt.pintaCentro txt
end sub

'-----------------------------------------------------------------------
sub player.novo
	z = 0
	vel = 0.03
	min_vel = 0.03
	acc = 0
	ang = 0
	angacc = 0
	ang_hor = 3 * (pi / 2)
	ang_ver = pi
	antigrav = false
	particula_z = 0
	flash = 0
	shield = 3
	hover = 2.9
	energia = 1
	energia_vis = energia
end sub

sub player.controle
	acc = 0
	antigrav = false

	if kev.confereTecla(kev.K_LEFT, kev.DOWN) then ang = angadd(ang, angacc * fps.getFator)
	if kev.confereTecla(kev.K_RIGHT, kev.DOWN) then ang = angadd(ang, angacc * -1 * fps.getFator)
	if kev.confereTecla(kev.K_UP, kev.DOWN) and shield > 0 then acc = 0.0015
	if kev.confereTecla(kev.K_DOWN, kev.DOWN) then acc = -0.004
	if kev.confereTecla(kev.K_BOOST, kev.DOWN) and hover > 0 then antigrav = true
end sub

sub player.fisica
	dim as double last_vel = vel

	'movimento
	vel = vel + (acc * fps.getFator)
	vel = vel * (0.997 ^ fps.getFator)
	
	if vel > 0.5 then vel = 0.5
	if vel < 0.03 then vel = 0.03
	
	z = z - (vel * fps.getFator)
	while z < 0
		z = z + 1
		tunel.push
		colisao
	wend
	
	if not antigrav then
		'centrífuga
		ang = angadd(ang, tunel.r(1).ang_hor * vel * 1.2 * fps.getFator)
	
		'gravidade	
		if ang < 0 then
			ang = angadd(ang, vel * 0.04 * sin(ang) * fps.getFator)
			ang = angadd(ang, 0.01 * sin(ang) * fps.getFator)
		else
			ang = angadd(ang, vel * -0.04 * sin(ang) * fps.getFator)
			ang = angadd(ang, -0.01 * sin(ang) * fps.getFator)
		end if
	end if
	
	'aceleração lateral
	angacc = 0.02
	if vel < 0.2 then angacc = angacc * (vel / 0.2)
	if antigrav then
		angacc = angacc * 2
		
		hover = hover - 0.01 * fps.getFator
		if hover < 0 then hover = 0
	end if
	
	'gira fundo
	ang_hor = angadd(ang_hor, tunel.r(0).ang_hor * vel * fps.getFator)
	ang_ver = angadd(ang_ver, tunel.r(0).ang_ver * vel * fps.getFator)
	
	'offset de partículas
	particula_z = particula_z + ((0.01 + (vel * 0.1)) * fps.getFator)
	while particula_z > 1
		particula_z = particula_z - 1
	wend
	
	'flash
	if flash > 0 then
		flash = flash - (0.02 * fps.getFator)
	end if
	
	'velocidade mínima
	min_vel = min_vel + (0.00005 * fps.getFator)
	if min_vel > 0.475 then min_vel = 0.475
	
	if shield = 0 and min_vel > 0 then min_vel = min_vel - (0.001 * fps.getFator)
	
	'energia
	if energia > 0 then 
		if vel < min_vel then
			energia = energia - (0.001 * fps.getFator)
			
			if energia < 0 then
				energia = 0
				shield = 0
				msg.nova "energy depleted!"
			else
				if last_vel >= min_vel and shield > 0 then
					select case int(rnd * 3)
						case 0: msg.nova "pick it up!"
						case 1: msg.nova "hurry up!"
						case else: msg.nova "step on it!"
					end select
				end if
			end if
		elseif energia < 1 then
			energia = energia + (0.001 * fps.getFator)
			
			if energia > 1 then energia = 1
		end if
	
		if energia > 2 then energia = 2
	end if
	
	if energia_vis > energia then
		energia_vis = energia
	elseif energia_vis < energia then
		energia_vis = energia_vis + (0.001 * fps.getFator)
		if energia_vis > energia then energia_vis = energia
	end if
end sub

sub player.colisao
	dim as _bool_ colide = false

	'funil horizontal
	if tunel.r(1).objeto = 1 then
		if not(angdif(ang, 0) < 0.3 or angdif(ang, pi) < 0.3) then colide = true
		
	'funil vertical
	elseif tunel.r(1).objeto = 2 then
		if not(angdif(ang, pi / 2) < 0.3 or angdif(ang, pi * 1.5) < 0.3) then colide = true
		
	'funil giratório
	elseif tunel.r(1).objeto = 3 then
		dim as double teste1, teste2
		teste1 = tunel.r(1).obj_ang
		teste2 = angadd(tunel.r(1).obj_ang, pi)
		
		if not(angdif(ang, teste1) < 0.3 or angdif(ang, teste2) < 0.3) then	colide = true
	
	'power-up
	elseif tunel.r(1).objeto >= 4 then
		if angdif(ang, tunel.r(1).obj_ang) < 0.35 then
			select case tunel.r(1).objeto
				case 4
					shield = shield + 1
					if shield > 5 then shield = 5
					
					msg.nova "shield advance"
					
				case 5
					hover = hover + 1
					if hover > 4.9 then hover = 4.9
					
					msg.nova "hover advance"
					
				case 6
					energia = energia + (1 / 8)
					if energia > 2 then energia = 2
				
					msg.nova "energy bonus"
				
				case 7
					msg.nova "mystery!"
			end select
			
			tunel.r(1).objeto = 0
		end if
	end if
	
	if colide then
		shield = shield - 1
		if shield < 0 then
			shield = 0
		elseif shield = 0 then
			energia = 0
			msg.nova "shield depleted!"
		end if
		
		flash = 1
	end if
end sub

sub player.pinta
	'cria coordenadas
	dim as coord3d b1, b2, b3, b4, w1, w2, w3, w4
	b1.x = 0.3:		b1.y = 1 - 0.2:		b1.z = 1
	b2.x = 0.4:		b2.y = 1 - 0.4:		b2.z = 0.5
	b3.x = 0.2:		b3.y = 1 - 0.5:		b3.z = 0.35
	b4.x = 0.4:		b4.y = 1 - 0.4:		b4.z = 0
	w1.x = 0.4:		w1.y = 1 - 0.4:		w1.z = 0.45
	w2.x = 1.2:		w2.y = 1 - 0.3:		w2.z = 0.2
	w3.x = 1.2:		w3.y = 1 - 0.3:		w3.z = 0
	w4.x = 1.3:		w4.y = 1 - 0.8:		w4.z = -0.1

	'pinta
	glPushMatrix
		glTranslatef 0, 0, 2
		glRotatef (ang * 180 / pi), 0, 0, 1
		
		glPushMatrix
			glTranslatef 0, -0.4, -0.5
			glRotatef sin(ang) * -25, 0, 1, 0
			glScalef 0.2, -0.6, 1
			
			'sombra
			hsva 0, 0, 1, 0.3
			gfx.sombra.aplica
			glBegin GL_QUADS
				glTexCoord2i 0, 0
				glVertex3f w2.x, 1, b1.z
				glTexCoord2i 0, 1
				glVertex3f w2.x, 1, w3.z
				glTexCoord2i 1, 1
				glVertex3f w2.x * -1, 1, w3.z
				glTexCoord2i 1, 0
				glVertex3f w2.x * -1, 1, b1.z
			glEnd
			
			'texturas
			dim as single tx, ty
			tx = 0
			
			if shield <= 0 then
				ty = 0.75
			else
				select case particula_z
					case is < 0.25
						ty = 0
					case 0.25 to 0.5
						ty = 0.25
					case 0.5 to 0.75
						ty = 0.5
					case else
						ty = 0.25
				end select
			end if
			
			glEnable GL_DEPTH_TEST
			glEnable GL_CULL_FACE
			glEnable GL_LIGHTING
			
			hsv 0, 0, 1
			gfx.ship.aplica
			glBegin GL_TRIANGLES
				'asa grande
				glNormal3f 1, -0.5, 0
				
				glTexCoord2f 1, 1
				glVertex3f w2.x, w2.y, w2.z
				glTexCoord2f 0.5, 1
				glVertex3f w3.x, w3.y, w3.z
				glTexCoord2f 0.5, 0
				glVertex3f w4.x, w4.y, w4.z
				
				glTexCoord2f 1, 1
				glVertex3f w2.x * -1, w2.y, w2.z
				glTexCoord2f 0.5, 1
				glVertex3f w3.x * -1, w3.y, w3.z
				glTexCoord2f 0.5, 0
				glVertex3f w4.x * -1, w4.y, w4.z
				
				'asa grande (flip)
				glNormal3f -1, -0.5, 0
				
				glTexCoord2f 0.5, 1
				glVertex3f w3.x, w3.y, w3.z
				glTexCoord2f 1, 1
				glVertex3f w2.x, w2.y, w2.z
				glTexCoord2f 0.5, 0
				glVertex3f w4.x, w4.y, w4.z
				
				glTexCoord2f 0.5, 1
				glVertex3f w3.x * -1, w3.y, w3.z
				glTexCoord2f 1, 1
				glVertex3f w2.x * -1, w2.y, w2.z
				glTexCoord2f 0.5, 0
				glVertex3f w4.x * -1, w4.y, w4.z
				
				'traseira
				glNormal3f 0, 0, -1
				
				glTexCoord2f tx + 0.05, ty
				glVertex3f b4.x, b4.y, b4.z
				glTexCoord2f tx + 0.05, ty + 0.25
				glVertex3f b4.x, 0.8, b4.z
				glTexCoord2f tx + 0.45, ty
				glVertex3f b4.x * -1, b4.y, b4.z
					
				glTexCoord2f tx + 0.45, ty
				glVertex3f b4.x * -1, b4.y, b4.z
				glTexCoord2f tx + 0.05, ty + 0.25
				glVertex3f b4.x, 0.8, b4.z
				glTexCoord2f tx + 0.45, ty + 0.25
				glVertex3f b4.x * -1, 0.8, b4.z
				
				'grill
				glNormal3f 0, 0.4, 1
				
				glTexCoord2f 0.71875, 0
				glVertex3f b1.x, b1.y, b1.z
				glTexCoord2f 1, 0
				glVertex3f b1.x * -1, b1.y, b1.z
				glTexCoord2f 0.71875, 0.171875
				glVertex3f b1.x, 0.9, b1.z - 0.05
				
				glTexCoord2f 0.71875, 0.171875
				glVertex3f b1.x, 0.9, b1.z  - 0.05
				glTexCoord2f 1, 0
				glVertex3f b1.x * -1, b1.y, b1.z
				glTexCoord2f 1, 0.171875
				glVertex3f b1.x * -1, 0.9, b1.z - 0.05
				
				'small patch
				glNormal3f 1, 0, 0
				
				glTexCoord2f 1, 0.21875
				glVertex3f b2.x, b2.y, b2.z
				glTexCoord2f 1, 0.578125
				glVertex3f b2.x, 0.8, b2.z
				glTexCoord2f 0.890625, 0.21875
				glVertex3f w1.x, w1.y, w1.z
				
				glTexCoord2f 0.890625, 0.21875
				glVertex3f w1.x, w1.y, w1.z
				glTexCoord2f 1, 0.578125
				glVertex3f b2.x, 0.8, b2.z
				glTexCoord2f 0.890625, 0.578125
				glVertex3f w1.x, 0.8, w1.z
				
				glNormal3f -1, 0, 0
				
				glTexCoord2f 1, 0.578125
				glVertex3f b2.x * -1, 0.8, b2.z
				glTexCoord2f 1, 0.21875
				glVertex3f b2.x * -1, b2.y, b2.z
				glTexCoord2f 0.890625, 0.21875
				glVertex3f w1.x * -1, w1.y, w1.z
				
				glTexCoord2f 1, 0.578125
				glVertex3f b2.x * -1, 0.8, b2.z
				glTexCoord2f 0.890625, 0.21875
				glVertex3f w1.x * -1, w1.y, w1.z
				glTexCoord2f 0.890625, 0.578125
				glVertex3f w1.x * -1, 0.8, w1.z
			
				'plain
				glTexCoord2f 0.015625, 0.015625 'pixel 1, 1
				
				'capô
				glNormal3f 0, -1, 0.2
				
				glVertex3f b1.x, b1.y, b1.z
				glVertex3f b2.x, b2.y, b2.z
				glVertex3f b1.x * -1, b1.y, b1.z
				
				glVertex3f b1.x * -1, b1.y, b1.z
				glVertex3f b2.x, b2.y, b2.z
				glVertex3f b2.x * -1, b2.y, b2.z
				
				'asa top
				glNormal3f 0.3, -1, 0
				
				glVertex3f w1.x, w1.y, w1.z
				glVertex3f w2.x, w2.y, w2.z
				glVertex3f b4.x, b4.y, b4.z
				
				glVertex3f b4.x, b4.y, b4.z
				glVertex3f w2.x, w2.y, w2.z
				glVertex3f w3.x, w3.y, w3.z
				
				glNormal3f -0.3, -1, 0
				
				glVertex3f w2.x * -1, w2.y, w2.z
				glVertex3f w1.x * -1, w1.y, w1.z
				glVertex3f b4.x * -1, b4.y, b4.z
				
				glVertex3f w2.x * -1, w2.y, w2.z
				glVertex3f b4.x * -1, b4.y, b4.z
				glVertex3f w3.x * -1, w3.y, w3.z
				
				'asa bottom
				glNormal3f 0.3, 1, 0
				
				glVertex3f w3.x, w3.y, w3.z
				glVertex3f w2.x, w2.y, w2.z
				glVertex3f b4.x, 0.8, b4.z
				
				glVertex3f w2.x, w2.y, w2.z
				glVertex3f w1.x, 0.8, w1.z
				glVertex3f b4.x, 0.8, b4.z
				
				glNormal3f -0.3, 1, 0
				
				glVertex3f w2.x * -1, w2.y, w2.z
				glVertex3f w3.x * -1, w3.y, w3.z
				glVertex3f b4.x * -1, 0.8, b4.z
				
				glVertex3f w1.x * -1, 0.8, w1.z
				glVertex3f w2.x * -1, w2.y, w2.z
				glVertex3f b4.x * -1, 0.8, b4.z
				
				'asa traseira
				glNormal3f 0, 0, -1
				
				glVertex3f w3.x, w3.y, w3.z
				glVertex3f b4.x, 0.8, b4.z
				glVertex3f b4.x, b4.y, b4.z
				
				glVertex3f b4.x * -1, 0.8, b4.z
				glVertex3f w3.x * -1, w3.y, w3.z
				glVertex3f b4.x * -1, b4.y, b4.z
				
				'bico lado
				glNormal3f 1, 0, 0
				
				glVertex3f b1.x, b1.y, b1.z
				glVertex3f b1.x, 0.9, b1.z  - 0.05
				glVertex3f b2.x, b2.y, b2.z
				
				glVertex3f b1.x, 0.9, b1.z  - 0.05
				glVertex3f b2.x, 0.8, b2.z
				glVertex3f b2.x, b2.y, b2.z
				
				glNormal3f -1, 0, 0
				
				glVertex3f b1.x * -1, 0.9, b1.z  - 0.05
				glVertex3f b1.x * -1, b1.y, b1.z
				glVertex3f b2.x * -1, b2.y, b2.z
				
				glVertex3f b2.x * -1, 0.8, b2.z
				glVertex3f b1.x * -1, 0.9, b1.z  - 0.05
				glVertex3f b2.x * -1, b2.y, b2.z
				
				'grill asa
				glNormal3f 0.8, 0, 1
				
				glVertex3f w1.x, w1.y, w1.z
				glVertex3f w1.x, 0.8, w1.z
				glVertex3f w2.x, w2.y, w2.z
				
				glNormal3f -0.8, 0, 1
				
				glVertex3f w1.x * -1, 0.8, w1.z
				glVertex3f w1.x * -1, w1.y, w1.z
				glVertex3f w2.x * -1, w2.y, w2.z
			
				'vidro traseiro
				glNormal3f 0, -1, -0.2
				
				glVertex3f b3.x, b3.y, b3.z
				glVertex3f b4.x, b4.y, b4.z
				glvertex3f b3.x * -1, b3.y, b3.z
				
				glvertex3f b3.x * -1, b3.y, b3.z
				glVertex3f b4.x, b4.y, b4.z
				glVertex3f b4.x * -1, b4.y, b4.z
			
				'vidro dianteiro
				glTexCoord2f 0.640625, 0.03125
				
				glNormal3f 0, -1, 0.5
				
				glVertex3f b2.x, b2.y, b2.z
				glVertex3f b3.x, b3.y, b3.z
				glVertex3f b2.x * -1, b2.y, b2.z
				
				glVertex3f b2.x * -1, b2.y, b2.z
				glVertex3f b3.x, b3.y, b3.z
				glVertex3f b3.x * -1, b3.y, b3.z
				
				'vidro lateral
				glNormal3f 1, -0.2, 0
				
				glVertex3f b2.x, b2.y, b2.z
				glVertex3f b4.x, b4.y, b4.z
				glVertex3f b3.x, b3.y, b3.z
				
				glNormal3f -1, -0.2, 0
				
				glVertex3f b4.x * -1, b4.y, b4.z
				glVertex3f b2.x * -1, b2.y, b2.z
				glVertex3f b3.x * -1, b3.y, b3.z
			glEnd
			
			glDisable GL_CULL_FACE
			glDisable GL_LIGHTING
			
			'motor antigrav / sombra
			dim d as double
			
			if antigrav then
				_notex_
				glBegin GL_TRIANGLE_FAN
					setPal pal.hover
					glVertex3f 0, 1, 0.3
					
					for d = 0 to (2 * pi) + (pi / 8) step (pi / 8)
						setPalA pal.hover, 0
						glVertex3f 1.5 * cos(d), 1, 0.3 + (0.5 * sin(d))
					next d
				glEnd
			end if
			
			glDisable GL_DEPTH_TEST
			
			'partículas
			if shield > 0 then
				dim as double tam = 0.02
				gfx.particula.aplica
			
				for d = 0 to 1 step 0.025
					dim zz as double
					zz = frac(particula_z + d)
					
					dim particula as coord3d
					particula.x = sin(d * pi * 12) * (zz + 0.1)
					particula.y = 0.72 + (sin(d * pi * 2) * zz * 0.5)
					particula.z = 0 - zz
					
					hsva d * 60, 1, 1, 1 - (zz ^ 3)
					
					glPushMatrix
						glTranslatef particula.x, particula.y, particula.z
						glScalef 3, 1, 1
						glRotatef 360 * zz, 0, 0, 1
						
						glBegin GL_QUADS
							glTexCoord2i 0, 0
							glVertex3f -tam, -tam, 0
							
							glTexCoord2i 0, 1
							glVertex3f -tam, tam, 0
							
							glTexCoord2i 1, 1
							glVertex3f tam, tam, 0
							
							glTexCoord2i 1, 0
							glVertex3f tam, -tam, 0
						glEnd
					glPopMatrix
				next d
			end if
		glPopMatrix
	glPopMatrix
end sub

sub player.pintaHud
	'modifica fonte
	fnt.setSombra true
	
	dim as integer temp_dx, temp_dy
	temp_dx = _dx_
	temp_dy = _dy_
	_dx_ = 6
	_dy_ = 9
	
	'velocímetro
	glPushMatrix
		if h <= 480 then
			glTranslatef w - 64, h - 64 - 16, 0
			glScalef 1, 1, 0
		else
			glTranslatef w - 128, h - 128 - 32, 0
			glScalef 2, 2, 0
		end if
	
		gfx.hud.aplica
	
		'shield e hover
		dim as integer i, valor
			
		for i = 1 to 2
			glPushMatrix
				if i = 1 then
					valor = p.shield
					setPal pal.shield
					glScalef -1, 1, 1
				else
					if p.hover > 0 then
						valor = int(p.hover) + 1
					else
						valor = 0
					end if
					setPal pal.hover
				end if
			
				if valor > 0 then
					dim as double tx, ty
							
					select case valor
						case 4, 5
							tx = 0
							ty = 0
						case 3
							tx = 0.5
							ty = 0
						case 2
							tx = 0
							ty = 0.5
						case else
							tx = 0.5
							ty = 0.5
					end select	
				
					if valor = 4 then
						glBegin GL_QUADS
							glTexCoord2f tx + 0.5, 15 / 128
							glVertex2f 0, 15
							
							glTexCoord2f tx + 0.5, ty + 0.5
							glVertex2f 0, 64
							
							glTexCoord2f tx, ty + 0.5
							glVertex2f 64, 64
							
							glTexCoord2f tx, 15 / 128
							glVertex2f 64, 15
						glEnd
					else
						glBegin GL_QUADS
							glTexCoord2f tx + 0.5, ty
							glVertex2f 0, 0
							
							glTexCoord2f tx + 0.5, ty + 0.5
							glVertex2f 0, 64
							
							glTexCoord2f tx, ty + 0.5
							glVertex2f 64, 64
							
							glTexCoord2f tx, ty
							glVertex2f 64, 0
						glEnd
					end if
				end if
			glPopMatrix
		next i
		
		'anéis
		dim d as double
		
		glPushMatrix
			glScalef 1, 1.04, 0
			glTranslatef 0, 30, 0
				
			'velocidade atual
			_notex_
			hsva 0, 0, 1, 0.8		
		
			glBegin GL_TRIANGLE_STRIP
				for d = 0 to (p.vel * 1.5 * 2 * pi) step (pi / 32)
					glVertex2f cos(d) * 18, sin(d) * 18
					glVertex2f cos(d) * 22, sin(d) * 22
				next d
			glEnd
		
			'velocidade mínima
			hsva 0, 1, 1, 0.8
			
			glBegin GL_TRIANGLE_STRIP
				for d = 0 to (p.min_vel * 1.5 * 2 * pi) step (pi / 32)
					glVertex2f cos(d) * 23, sin(d) * 23
					glVertex2f cos(d) * 26, sin(d) * 26
				next d
			glEnd
		glPopMatrix
		
		'símbolos
		fnt.setCor 1, 1, 1
		fnt.pinta "^", -14 -3, 60
		fnt.pinta "&", 14 -3, 60
		fnt.pinta "%", -6 -3, 27
		
		'energia
		fnt.setCor 0.5, 1, 0.4
		
		valor = int(energia_vis * 50)
		if valor >= 100 then valor = 99
		fnt.pinta str(valor), -1, 28		
	glPopMatrix
	
	'restaura fonte
	_dx_ = temp_dx
	_dy_ = temp_dy
	
	'decalque de aviso
	if shield > 0 and (shield = 1 or valor < 25) then
		hsva 0, 0, 1, particula_z
		
		gfx.aviso.aplica
		glBegin GL_QUADS
			glTexCoord2i 0, 0
			glVertex2f w * 0.4, h * 0.35
			glTexCoord2i 0, 1
			glVertex2f w * 0.4, h * 0.65
			glTexCoord2i 1, 1
			glVertex2f w * 0.6, h * 0.65
			glTexCoord2i 1, 0
			glVertex2f w * 0.6, h * 0.35
		glEnd
	end if
end sub

sub player.sons
	if shield <= 0 then
		som.setVolume 0, 0
	else
		som.setVolume 0, 0.4
		som.setPitch 0, 0.4 + (vel * 2)
	end if
	
	som.setVolume 1, vel * 1.3
end sub

'-----------------------------------------------------------------------
sub menuObj.main
	'primeira opção: new game
	cursor = 0

	'cria objeto glut para planeta
	planeta_quad = gluNewQuadric
	gluQuadricTexture planeta_quad, GL_TRUE
	gluQuadricNormals planeta_quad, GLU_SMOOTH	
	
	planeta_ang = 0
	
	'fluxo
	do
		topo
		
		select case cursor
			case 0: jogo
			case 1: escolheCurso
			case 2: escolheNave
			case 3:
			case else: exit do
		end select
	loop
	
	'encerra
	gluDeleteQuadric planeta_quad
	fim
end sub

sub menuObj.topo
	'prepara cena
	randomize 2
	fps.reinicia
	estrelas.nova
	p.novo 'just in case
		
	dim fonte as coord3d
	fonte.x = -10
	fonte.y = 0
	fonte.z = -9
	
	glMatrixMode GL_MODELVIEW
	glLoadIdentity
	gfx.ilumina fonte, 0
	
	dim as double cursor_pisca = 0
	
	'loop do menu principal
	do
		gfx.menuHeader
		
		'controles
		if kev.confereTecla(kev.K_ESCOLHE, kev.PRESSED) then
			exit sub
		elseif kev.confereTecla(kev.K_UP, kev.PRESSED) then
			if cursor > 0 then
				cursor = cursor - 1
				cursor_pisca = 0
			end if
		elseif kev.confereTecla(kev.K_DOWN, kev.PRESSED) then
			if cursor < 4 then
				cursor = cursor + 1
				cursor_pisca = 0
			end if
		elseif kev.confereTecla(kev.K_RETORNA, kev.PRESSED) then
			cursor = 4
			exit sub
		end if
				
		'movimento
		planeta_ang = angadd(planeta_ang, 0.001 * fps.getFator)
		cursor_pisca = angadd(cursor_pisca, 0.1 * fps.getFator)
		
		'gráficos
		estrelas.pinta
		
		'planeta
		glEnable GL_DEPTH_TEST				
		glPushMatrix
			glScalef -1, 1, 1
			glTranslatef 4, 0, 9
			glRotatef 70, 1, 0, 0
			glRotatef -10, 0, 1, 0
			glRotatef planeta_ang / to_rad, 0, 0, 1
			
			glEnable GL_LIGHTING
			
			hsv 0, 0, 1
			gfx.planeta.aplica
			gluSphere(planeta_quad, 3, 16, 12)
			
			glDisable GL_LIGHTING
			
			'anel
			gfx.asteroides.aplica
			glBegin GL_TRIANGLE_STRIP
				dim as integer ty = 0				
				dim as double d
					
				for d = 0 to (2 * pi) + (pi / 16) step (pi / 16)
					glTexCoord2i 0, ty
					glVertex3f cos(d) * 3.5, sin(d) * 3.5, 0
					glTexCoord2i 2, ty
					glVertex3f cos(d) * 4.3, sin(d) * 4.3, 0
					
					ty = ty + 2
				next d
			glEnd			
		glPopMatrix			
		glDisable GL_DEPTH_TEST
		
		'texto
		dim texto (0 to 4) as string
		texto(0) = "new game"
		texto(1) = "select course"
		texto(2) = "select ship"
		texto(3) = "achievements"
		texto(4) = "exit to hyperspace"
		
		fnt.orto
		
		fnt.setSombra false
		
		fnt.setCor 1, 1, 1
		fnt.pintaTopo "   kosmo rally", 2
		fnt.setCor 0.9, 0.2, 0
		fnt.pintaTopo "   __________", 3
		
		dim i as integer
		for i = 0 to 4
			if i = cursor then
				fnt.setCor 1, 1, 1, (sin(cursor_pisca) / 2) + 0.5
			else
				fnt.setCor 1, 1, 1, 0.5
			end if
			
			fnt.pintaTopo "   " + texto(i), (i * 2) + 5
		next i
		
		fnt.setCor 1, 1, 1
		fnt.pintaTopo " &", (cursor * 2) + 5		
		
		fnt.restaura
		
		gfx.menuFooter
	loop
end sub

sub menuObj.escolheCurso

end sub

sub menuObj.escolheNave
	'prepara cena
	p.novo
	p.shield = 0
	
	dim fonte as coord3d
	fonte.x = 0
	fonte.y = 10
	fonte.z = 10
	
	glMatrixMode GL_MODELVIEW
	glLoadIdentity
	gfx.ilumina fonte, 0.2
	
	dim as double ang = 0
	
	'loop da cena
	do
		gfx.menuHeader
		
		'movimento
		ang = angadd(ang, 0.01 * fps.getFator)
		
		'bg
		fnt.orto
		_notex_
		glBegin GL_QUADS
			hsv 240, 0.8, 0.1
			glVertex2f w, 0
			glVertex2f 0, 0
			
			hsv 220, 0.4, 0.2
			glVertex2f 0, h
			glVertex2f w, h
		glEnd
		fnt.restaura
		
		'nave			
		glPushMatrix
			glRotatef ang / to_rad, 0, 1, 0
			glTranslatef 0, 0.6, -2
					
			p.fisica		
			p.pinta
		glPopMatrix
		
		gfx.menuFooter
	loop until kev.confereTecla(kev.K_ESCOLHE, kev.PRESSED) or kev.confereTecla(kev.K_RETORNA, kev.PRESSED)
end sub
