const as _bool_ FILTERING_ON = false

type rgbaval
	r as single
	g as single
	b as single
	a as single
end type

type textura
	private:
		surface as SDL_Surface ptr
		tex as GLuint
	
	public:
		declare sub carrega (arquivo as string, aplica_filtro as _bool_, aplica_mipmap as _bool_, aplica_clamp as _bool_)
		declare sub carrega (arquivo as string, aplica_mipmap as _bool_, aplica_clamp as _bool_)
		declare sub carrega (arquivo as string)
		declare sub aplica
		declare sub destroi
end type

type fonte
	private:
		folha as textura
		cor as rgbaval
		sombra as _bool_
		
		declare sub pintaChar (i as integer, j as integer, x as GLfloat, y as GLfloat)
			
	public:
		declare sub orto ()
		declare sub restaura ()
		declare sub carrega (arquivo as string)
		declare sub setCor (r as GLfloat, g as GLfloat, b as GLfloat)
		declare sub setCor (r as GLfloat, g as GLfloat, b as GLfloat, a as GLfloat)
		declare sub setCor (cor as rgbaval)
		declare sub setSombra (sombra as _bool_)	
		declare sub pinta (texto as string, x as GLfloat, y as GLfloat)
		declare sub pintaTopo (texto as string, i as integer)
		declare sub pintaTopoDir (texto as string, i as integer)
		declare sub pintaCentro (texto as string)
		declare sub pintaCentro (texto as string, i as integer)
		declare sub destroi
end type

'-----------------------------------------------------------------------
sub textura.carrega (arquivo as string, aplica_filtro as _bool_, aplica_mipmap as _bool_, aplica_clamp as _bool_)
	dim formato as GLenum
	dim max_anis as GLfloat

	'carrega imagem
	surface = SDL_LoadBMP(arquivo)
	if surface = NULL then
		print "Problema carregando textura: "; arquivo
		end
	end if
	
	'mapeia imagem para opengl
	glGenTextures 1, @tex
	
	'define parmetros
	glBindTexture GL_TEXTURE_2D, tex
	if aplica_filtro then
		glTexParameteri GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR
		glTexParameteri GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR
	else
		glTexParameteri GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST
		glTexParameteri GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST
	end if
	
	if aplica_mipmap and aplica_filtro then
		glGetFloatv GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, @max_anis
		glTexParameterf GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, max_anis
	
		glTexParameteri GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST
		glTexParameteri GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE
	end if
	
	if aplica_clamp then
		glTexParameteri GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT
		glTexParameteri GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP
	else
		glTexParameteri GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT
		glTexParameteri GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT
	end if
	
	if surface->format->BytesPerPixel = 4 then
		formato = GL_BGRA
	else
		formato = GL_BGR
	end if
	glTexImage2D GL_TEXTURE_2D, 0, surface->format->BytesPerPixel, surface->w, surface->h, _
								0, formato, GL_UNSIGNED_BYTE, surface->pixels
								
	'libera surface
	SDL_FreeSurface surface	
end sub

sub textura.carrega (arquivo as string, aplica_mipmap as _bool_, aplica_clamp as _bool_)
	carrega arquivo, FILTERING_ON, aplica_mipmap, aplica_clamp
end sub

sub textura.carrega (arquivo as string)
	carrega arquivo, FILTERING_ON, true, false
end sub

sub textura.aplica ()
	glBindTexture GL_TEXTURE_2D, tex
end sub

sub textura.destroi ()
	glDeleteTextures 1, @tex
end sub

'-----------------------------------------------------------------------
sub fonte.orto
	'configura projeo ortogonal
	glMatrixMode GL_PROJECTION
	glPushMatrix
	glLoadIdentity
	
	glOrtho 0, w, h, 0, -1, 1
	
	glMatrixMode GL_MODELVIEW
	glPushMatrix
	glLoadIdentity
end sub

sub fonte.restaura
	'restaura projeo 3D
	glMatrixMode GL_PROJECTION
	glPopMatrix
	glMatrixMode GL_MODELVIEW
	glPopMatrix
end sub

sub fonte.carrega (arquivo as string)
	folha.carrega arquivo, false, false, false
	
	'inicializa valores
	cor.r = 1
	cor.g = 1
	cor.b = 1
	sombra = true
end sub

sub fonte.setCor (r as GLfloat, g as GLfloat, b as GLfloat)
	cor.r = r
	cor.g = g
	cor.b = b
	cor.a = 1
end sub

sub fonte.setCor (r as GLfloat, g as GLfloat, b as GLfloat, a as GLfloat)
	cor.r = r
	cor.g = g
	cor.b = b
	cor.a = a
end sub

sub fonte.setCor (cor as rgbaval)
	this.cor = cor
end sub

sub fonte.setSombra (sombra as _bool_)
	this.sombra = sombra
end sub

sub fonte.pintaChar (i as integer, j as integer, x as GLfloat, y as GLfloat)
	dim as double z, tx, ty, td

	'delta da coordenada do texel
	td = 1 / 64
	
	'define plano do texto
	z = 0
	
	'desenha quad
	tx = td * 6 * i
	ty = td * 9 * j
				
	glColor4f cor.r, cor.g, cor.b, cor.a 'colore
	
	folha.aplica
	glBegin GL_QUADS
		glTexCoord2f tx, ty + (td * 9)
		glVertex3f x, y + _dy_, z
		
		glTexCoord2f tx + (td * 6), ty + (td * 9)
		glVertex3f x + _dx_, y + _dy_, z
		
		glTexCoord2f tx + (td * 6), ty
		glVertex3f x + _dx_, y , z
		
		glTexCoord2f tx, ty
		glVertex3f x, y, z
	glEnd
	
	glColor4f 1, 1, 1, 1 'restaura cor
end sub

sub fonte.pinta (texto as string, x as GLfloat, y as GLfloat)
	dim as integer i, j
			
	'trata string
	if len(texto) > 1 then
		dim as GLfloat tx = x
		dim char as string
		
		for i = 1 to len(texto)
			char = mid(texto, i, 1)
			
			pinta char, tx, y
			
			select case char
				case "i", "l", ".", ",", ":", "!", "'" 'caracteres finos
					tx = tx + (_dx_ / 3)
				case else
					tx = tx + _dx_
			end select
		next
	
	'trata caractere
	else
		'converte caixa
		texto = lcase(texto)
		
		'define coordenada
		select case texto
			case " "
				exit sub
			case "a" to "j"
				j = 0
				i = asc(texto) - asc("a")
			case "k" to "t"
				j = 1
				i = asc(texto) - asc("k")
			case "u" to "z"
				j = 2
				i = asc(texto) - asc("u")
			case "0" to "9"
				j = 3
				i = asc(texto) - asc("0")
			
			case "", "": j = 4: i = 0
			case "", "": j = 4: i = 1
			case "", "": j = 4: i = 2
			case "", "": j = 4: i = 3
			case "", "": j = 4: i = 4
			case "", "", "", "": j = 4: i = 5
			case "", "": j = 4: i = 6
			case "", "", "", "": j = 4: i = 7
			case "", "": j = 4: i = 8
			
			case "!": j = 5: i = 0
			case "?": j = 5: i = 1
			case ".": j = 5: i = 2
			case ",": j = 5: i = 3
			case ":": j = 5: i = 4
			case "/": j = 5: i = 5
			case "(": j = 5: i = 6
			case ")": j = 5: i = 7
			case "*": j = 5: i = 8
			case "-": j = 5: i = 9
			case "+": j = 6: i = 0
			case "<": j = 6: i = 1
			case ">": j = 6: i = 2
			case "#": j = 6: i = 3 'copyright
			case "@": j = 6: i = 4
			case "_": j = 6: i = 5
			case "$": j = 6: i = 6 'reticncias
			case "'": j = 6: i = 7
			case "|": j = 6: i = 8 'up arrow
			
			case "%": j = 2: i = 6 'energia
			case "^": j = 2: i = 7 'vida
			case "&": j = 2: i = 8 'hover
			
			case else: j = 5: i = 1
		end select
		
		'desenha caractere
		dim _1px_ as GLfloat
		dim tmp as rgbaval
		_1px_ = _dx_ / 6
		tmp = cor
		
		if sombra then
			setCor 0, 0, 0
			pintaChar i, j, x - _1px_, y - _1px_
			pintaChar i, j, x, y - _1px_
			pintaChar i, j, x + _1px_, y - _1px_
			pintaChar i, j, x - _1px_, y
			pintaChar i, j, x + _1px_, y
			pintaChar i, j, x - _1px_, y + _1px_
			pintaChar i, j, x, y + _1px_
			pintaChar i, j, x + _1px_, y + _1px_
		end if
		
		setCor tmp
		pintaChar i, j, x, y	
	end if
end sub

sub fonte.pintaTopo (texto as string, i as integer)
	pinta texto, _dx_, i * _dy_
end sub

sub fonte.pintaTopoDir (texto as string, i as integer)
	pinta texto, w - (len(texto) * _dx_) - _dx_, i * _dy_
end sub

sub fonte.pintaCentro (texto as string)
	pinta texto, (w - (len(texto) * _dx_)) / 2, (h - _dy_) / 2
end sub

sub fonte.pintaCentro (texto as string, i as integer)
	pinta texto, (w - (len(texto) * _dx_)) / 2, i * _dy_
end sub

sub fonte.destroi ()
	folha.destroi
end sub
